from typing import Union, Type, Text

import magic

import gb.entry


mime_entry = {"text": gb.entry.Text}


def guess_type(path: str) -> Union[Type[Text], str, None]:
    """Guess the type of a path and return the corresponding Entry for that
       type with its data loaded."""

    guess = magic.Magic(mime=True).from_file(path).split("/")[0]
    return mime_entry.get(guess, gb.entry.Binary)  # type: ignore



class Entry:
    """Gopher indexes contain entries of various types. This list is both
       sourced from RFC 1436:

       https://tools.ietf.org/html/rfc1436

       and the Wikipedia page for non-standard types:

       https://en.wikipedia.org/wiki/Gopher_(protocol)#Item_types"""

    host: str
    port: int

    def __init__(self, host: str, port: int, text: str, selector: str) -> None:
        self.host = host
        self.port = port

        self.text = text
        self.selector = selector


class Text(Entry):
    code = 0


class Directory(Entry):
    code = 1

    def __init__(self, host: str, port: int, text: str, selector: str) -> None:
        super().__init__(host, port, text, selector)

        self.text = text + "/"


class CCSO(Entry):
    code = 2


class Error(Entry):
    code = 3


class BinHex(Entry):
    code = 4


class DOS(Entry):
    code = 5


class UU(Entry):
    code = 6


class Search(Entry):
    code = 7


class Telnet(Entry):
    code = 8


class Binary(Entry):
    code = 9


class Mirror(Entry):
    code = "+"


class GIF(Entry):
    code = "g"


class Image(Entry):
    code = "I"


class Telnet3270(Entry):
    code = "T"


class HTML(Entry):
    code = "h"


class Information(Entry):
    code = "i"


class Sound(Entry):
    code = "s"



from typing import List

import gb.protocol
import gb.entry


class Document:
    """A document is an index of entries for a certain directory. It dictates
      how those entries are joined together and if any extra information is
      added."""

    entries: List[gb.entry.Entry]

    def __init__(self) -> None:
        self.entries = []

    def add_entry(self, entry: gb.entry.Entry) -> None:
        self.entries.append(entry)

    def __str__(self) -> str:
        return (
            gb.protocol.crlf.join(
                gb.protocol.template.format(e=e) for e in self.entries
            )
            + gb.protocol.crlf
        )



import logging

log = logging.getLogger(__name__)

crlf = "\r\n"
eof = "."

template = "{e.code}{e.text}\t{e.selector}\t{e.host}\t{e.port}"


def clean_selector(selector: str) -> str:
    """Strip the end off of a selector and normalize an empty one."""
    selector = selector.rstrip(crlf)

    return selector if selector != "" else "/"


def is_valid_selector(selector: str) -> bool:
    """Validate the selector according to gopher rules."""

    # TODO implement <TAB> denoted search

    # These characters are forbidden in gopher selectors and should immediately
    # terminate the connection
    return not any(forbidden in selector for forbidden in "\t\r\n")



import os


def relativize(base: str, path: str) -> str:
    """Join base and path making sure that the resulting path lies
       within the base. We do this by making path relative if it's an
       absolute path then verifying if it's still within the base after
       joining.

       We fail hard if the calculated path is not within base."""

    base = os.path.abspath(base)

    # The nif the path starts with a separator we remove that
    if path.startswith(os.path.sep):
        path = path[len(os.path.sep) :]

    dest = os.path.join(base, path)
    dest = os.path.abspath(dest)

    if os.path.commonprefix([base, dest]) != base:
        # TODO we want a better way to communicate this to our server
        # TODO so it can cleanly close the connection
        raise Exception("welp")

    return dest



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gb']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.0,<9.0', 'python-magic>=0.4.27,<0.5.0', 'tornado>=6.0,<7.0']

entry_points = \
{'console_scripts': ['gb = gb.__main__:main']}

setup_kwargs = {
    'name': 'gb',
    'version': '0.1.2',
    'description': 'A Python gopher server.',
    'long_description': '![gb logo, a gopher in a ball](https://src.tty.cat/supakeen/gb/raw/branch/master/doc/_static/logo-doc.png)\n\n# gb\n\n![rtd badge](https://readthedocs.org/projects/gb/badge/?version=latest) ![license badge](https://gb.readthedocs.io/en/latest/_static/license.svg) ![black badge](https://img.shields.io/badge/code%20style-black-000000.svg)\n\n## About\n\n`gb` or gopherball is a gopher server written in Python with the main goals of\nease of use and integration. The name gopherball is inspired by a recurring\ntheme in the Calvin & Hobbes comicbooks and a tongue in cheek reference of an\nalternative to the World Wide Web as we know it today.\n\n## Examples\nQuick examples to get you running.\n\n`gb --mode=implicit .` will start a gopher server on `127.0.0.1` port `7070` serving\na recursive index of files starting from the current directory.\n\n`gb --mode=implicit --magic .` will start `gb` in magic-mode on `127.0.0.1` port\n`7070`. Magic mode will make `gb` guess at filetypes.\n\n`gb --mode=implicit --host="127.1.1.1" --port 1025 .` will start `gb` in implicit\nmode on the chosen ip and port. Note that using ports under 1024 requires\nsuperuser permissions!\n\n## Technology\n`gb` is written with the help of Python 3.9 and higher and the Tornado\nframework for its networking.\n\n## Modes\n`gb` has one main mode of operation that is commonly used. More modes are\nplanned for the future.\n\n### implicit\nImplicit mode serves a directory recursively. Indexes are automatically\ngenerated and text files are served to the client. Data files are also\nsupported.\n\n## Magic\n`gb` will serve all non-directories as type 9 files, these are non-readable\nfiles and most clients will prompt for download. Turning on magic with\n`--magic` will let `gb` try to determine the correct filetypes.\n\n## Contributing\nThe source code for `gb` lives on my Gitea where you can also submit issues and\npull requests. It mostly needs help by people with the ability to test in\nvarious clients and libraries that might still support the gopher protocol.\n',
    'author': 'Simon de Vlieger',
    'author_email': 'cmdr@supakeen.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://src.tty.cat/supakeen/gb',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4',
}


setup(**setup_kwargs)
